@@ -138,6 +138,12 @@ call_returned (GObject *object,
   ret = g_dbus_connection_call_finish (bus, result, &error);
   if (error)
     {
+      if (call->cancelled_id)
+        {
+          g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+          call->cancelled_id = 0;
+        }
+
       g_task_return_error (call->task, error);
       account_call_free (call);
     }
@@ -151,7 +157,7 @@ get_user_information (AccountCall *call)
   GCancellable *cancellable;
 
   if (call->parent_handle == NULL)
-    {   
+    {
       call->parent->parent_export (call->parent, parent_exported, call);
       return;
     }
@@ -240,9 +246,10 @@ xdp_portal_get_user_information (XdpPortal *portal,
  * @result: a [iface@Gio.AsyncResult]
  * @error: return location for an error
  *
- * Finishes the get-user-information request, and returns
- * the result in the form of a [struct@GLib.Variant] dictionary containing
- * the following fields:
+ * Finishes the get-user-information request.
+ *
+ * Returns the result in the form of a [struct@GLib.Variant] dictionary
+ * containing the following fields:
  *
  * - id `s`: the user ID
  * - name `s`: the users real name
