@@ -43,7 +43,7 @@ inhibit_call_free (InhibitCall *call)
       call->parent->parent_unexport (call->parent);
       xdp_parent_free (call->parent);
     }
- g_free (call->parent_handle);
+  g_free (call->parent_handle);
 
   if (call->signal_id)
     g_dbus_connection_signal_unsubscribe (call->portal->bus, call->signal_id);
@@ -84,6 +84,12 @@ call_returned (GObject *object,
   ret = g_dbus_connection_call_finish (G_DBUS_CONNECTION (object), result, &error);
   if (error)
     {
+      if (call->cancelled_id)
+        {
+          g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+          call->cancelled_id = 0;
+        }
+
       g_hash_table_remove (call->portal->inhibit_handles, GINT_TO_POINTER (call->id));
       g_task_return_error (call->task, error);
       inhibit_call_free (call);
@@ -103,6 +109,12 @@ response_received (GDBusConnection *bus,
   guint32 response;
   g_autoptr(GVariant) ret = NULL;
 
+  if (call->cancelled_id)
+    {
+      g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+      call->cancelled_id = 0;
+    }
+
   g_variant_get (parameters, "(u@a{sv})", &response, &ret);
 
   if (response == 0)
@@ -127,7 +139,7 @@ inhibit_cancelled_cb (GCancellable *cancellable,
 {
   InhibitCall *call = data;
 
-g_debug ("inhibit cancelled, calling Close");
+  g_debug ("inhibit cancelled, calling Close");
   g_dbus_connection_call (call->portal->bus,
                           PORTAL_BUS_NAME,
                           call->request_path,
@@ -258,9 +270,10 @@ xdp_portal_session_inhibit (XdpPortal            *portal,
  * @result: a [iface@Gio.AsyncResult]
  * @error: return location for an error
  *
- * Finishes the inhbit request, and returns the ID of the
- * inhibition as a positive integer. The ID can be passed to
- * [method@Portal.session_uninhibit] to undo the inhibition.
+ * Finishes the inhbit request.
+ *
+ * Returns the ID of the inhibition as a positive integer. The ID can be passed
+ * to [method@Portal.session_uninhibit] to undo the inhibition.
  *
  * Returns: the ID of the inhibition, or -1 if there was an error
  */
@@ -476,6 +489,12 @@ create_returned (GObject *object,
   ret = g_dbus_connection_call_finish (G_DBUS_CONNECTION (object), result, &error);
   if (error)
     {
+      if (call->cancelled_id)
+        {
+          g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+          call->cancelled_id = 0;
+        }
+
       g_task_return_error (call->task, error);
       create_monitor_call_free (call);
     }
@@ -588,8 +607,9 @@ xdp_portal_session_monitor_start (XdpPortal *portal,
  * @result: a [iface@Gio.AsyncResult]
  * @error: return location for an error
  *
- * Finishes a session-monitor request, and returns
- * the result in the form of boolean.
+ * Finishes a session-monitor request.
+ *
+ * Returns the result in the form of boolean.
  *
  * Returns: `TRUE` if the request succeeded
  */
