@@ -84,6 +84,12 @@ response_received (GDBusConnection *bus,
   guint32 response;
   g_autoptr(GVariant) ret = NULL;
 
+  if (call->cancelled_id)
+    {
+      g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+      call->cancelled_id = 0;
+    }
+
   g_variant_get (parameters, "(u@a{sv})", &response, &ret);
 
   if (response == 0)
@@ -148,6 +154,12 @@ call_returned (GObject *object,
 
   if (error)
     {
+      if (call->cancelled_id)
+        {
+          g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+          call->cancelled_id = 0;
+        }
+
       g_task_return_error (call->task, error);
       open_call_free (call);
     }
@@ -301,8 +313,9 @@ xdp_portal_open_uri (XdpPortal *portal,
  * @result: a [iface@Gio.AsyncResult]
  * @error: return location for an error
  *
- * Finishes the open-uri request, and returns
- * the result in the form of a boolean.
+ * Finishes the open-uri request.
+ *
+ * Returns the result in the form of a boolean.
  *
  * Returns: `TRUE` if the call succeeded
  */
@@ -328,8 +341,9 @@ xdp_portal_open_uri_finish (XdpPortal *portal,
  * @callback: (scope async): a callback to call when the request is done
  * @data: (closure): data to pass to @callback
  *
- * Opens the directory containing the file specified by the @uri. which
- * must be a file: uri pointing to a file that the application has access
+ * Opens the directory containing the file specified by the @uri.
+ *
+ * which must be a file: uri pointing to a file that the application has access
  * to.
  */
 void
@@ -368,8 +382,9 @@ xdp_portal_open_directory (XdpPortal *portal,
  * @result: a [iface@Gio.AsyncResult]
  * @error: return location for an error
  *
- * Finishes the open-directory request, and returns
- * the result in the form of a boolean.
+ * Finishes the open-directory request.
+ *
+ * Returns the result in the form of a boolean.
  *
  * Returns: `TRUE` if the call succeeded
  */
