@@ -83,6 +83,12 @@ response_received (GDBusConnection *bus,
   guint32 response;
   g_autoptr(GVariant) ret = NULL;
 
+  if (call->cancelled_id)
+    {
+      g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+      call->cancelled_id = 0;
+    }
+
   g_variant_get (parameters, "(u@a{sv})", &response, &ret);
 
   if (response == 0)
@@ -147,6 +153,12 @@ call_returned (GObject *object,
 
   if (error)
     {
+      if (call->cancelled_id)
+        {
+          g_signal_handler_disconnect (g_task_get_cancellable (call->task), call->cancelled_id);
+          call->cancelled_id = 0;
+        }
+
       g_task_return_error (call->task, error);
       wallpaper_call_free (call);
     }
@@ -312,8 +324,9 @@ xdp_portal_set_wallpaper (XdpPortal           *portal,
  * @result: a [iface@Gio.AsyncResult]
  * @error: return location for an error
  *
- * Finishes the open-uri request, and returns
- * the result in the form of a boolean.
+ * Finishes the open-uri request.
+ *
+ * Returns the result in the form of a boolean.
  *
  * Returns: `TRUE` if the call succeeded
  */
